@@ -1,8 +1,8 @@
 import pandas as pd
-import yaml
 import numpy as np
+import math 
 
-from dreem.aggregate.resources.get_attributes import read_sample_attributes
+from ..resources.get_attributes import read_sample_attributes
 
 def get_samples_info(df_samples, sample, verbose= False):
     if verbose: print(f"Adding samples info for {sample}")
@@ -30,10 +30,19 @@ def get_samples_info(df_samples, sample, verbose= False):
 def get_library_info(df_library, reference, verbose= False):
     if verbose: print(f"Adding library info for {reference}")
 
+    if df_library is None:
+        return {},{}
     # Sanity check
     df_library = df_library[df_library['reference']==reference]
+    
+    section_translation = {str(ss)+'-'+str(se):s for s,ss,se in zip(df_library['section'],df_library['section_start'],df_library['section_end'])}
 
     df_library.drop(columns = [c for c in df_library.columns if c in ['section_start','section_end','section','reference']], inplace=True)
     
-    return df_library.iloc[0].to_dict()
-
+    d = df_library.iloc[0].to_dict()
+    for k,v in d.copy().items():
+        if type(v) is float:
+            if math.isnan(v):
+                del d[k]
+    
+    return d, section_translation
