@@ -2,7 +2,7 @@ import numpy as np
 import scipy.stats
 from scipy.optimize import newton_krylov
 
-from dreem.util.cli import *
+from ..util.cli import *
 
 ## ------------- Utility functions ------------- ##
 
@@ -54,12 +54,24 @@ class EMclustering:
         
     read_hist: array (N)
         Number of reads per bitvector.
-        
+
+    bases_to_keep: array (D)
+        Array of booleans indicating which bases to keep.
+    
+    sequence: str
+        Reference sequence.
+
+    max_procs: int
+        Maximum number of processes to use.
+
     min_iter: int
         Minimum number of iterations.
+
+    max_iter: int
+        Maximum number of iterations.
         
-    convergence_eps: float
-        Convergence threshold. When the difference between the log-likelihood of two consecutive iterations is lower than convergence_eps, the algorithm stops.
+    convergence_cutoff: float
+        Convergence threshold. When the difference between the log-likelihood of two consecutive iterations is lower than convergence_cutoff, the algorithm stops.
         
     Key methods:
     ------------
@@ -70,7 +82,7 @@ class EMclustering:
     
     """
     
-    def __init__(self, bv, K, read_hist, bases_to_keep, sequence, n_cpus: int, max_clusters:int=MAX_CLUSTERS, signal_thresh:float=SIGNAL_THRESH, info_thresh:float=INFO_THRESH, include_g_u:bool=INCLUDE_G_U, include_del:bool=INCLUDE_DEL, min_reads:int=MIN_READS, min_iter:int=MIN_ITER, convergence_cutoff:float=CONVERGENCE_CUTOFF, num_runs:int=NUM_RUNS, verbose:bool=VERBOSE):
+    def __init__(self, bv, K, read_hist, bases_to_keep, sequence, max_procs: int, max_clusters: int, signal_thresh: float, include_gu: bool, include_del: bool, min_reads: int, min_iter: int, max_iter:int, convergence_cutoff: float, num_runs: int, verbose: bool = True):
         self.bv = bv
         self.sparse_mu = np.zeros((K, len(sequence)))
         self.bases_to_keep = bases_to_keep
@@ -80,20 +92,22 @@ class EMclustering:
         self.D = bv.shape[1]
         
         self.min_iter = min_iter
-        self.cpus = n_cpus
+        self.max_iter = max_iter
 
-        self.convergence_eps = convergence_cutoff
+        self.convergence_cutoff = convergence_cutoff
+
+        self.verbose = verbose
     
     def expectation(self, mu, pi):
         """
         Run the Expectation step of the EM algorithm - calc log like
         Args:
-            X (EM_Class object): Contains the list of bit vectors
+            X (EM_Class object): Contains the list of bit mut_vectors
             K (int): Number of clusters
             mu (list): DMS reactivities in each cluster
             pi (list): Proportion of each cluster
             calc_inds (list): Indices to split the matrix at
-            n_cpus (int): Number of cpus to use
+            max_procs (int): Number of processes to use
             max_clusters (int): Maximum number of clusters
             signal_thresh (float): Threshold for signal
             info_thresh (float): Threshold for information
@@ -142,7 +156,7 @@ class EMclustering:
         """
         Run the Maximization step of the EM algorithm - update mu and pi
         Args:
-            X (EM_Class object): Contains the list of bit vectors
+            X (EM_Class object): Contains the list of bit mut_vectors
             K (int): Number of clusters
             mu (list): DMS reactivities in each cluster
             resps (list): Responsibity of each cluster
@@ -226,7 +240,9 @@ class EMclustering:
         log_like_list, mu_list, obs_pi_list, real_pi_list = [], [], [], []
 
         while not converged:  # Each iteration of the EM algorithm
-            # print('Iteration:', iter, '|', np.min(mu), np.max(mu))
+
+            if self.verbose:
+                print('Iteration:', iter, '|', np.min(mu), np.max(mu))
 
             # Expectation step
             (resps, log_like, denom) = self.expectation(mu, obs_pi)
@@ -249,9 +265,13 @@ class EMclustering:
                 if iter >= self.min_iter:  # At least min iterations has run
                     prev_loglike = log_like_list[-2]
                     diff = log_like - prev_loglike
-                    if diff <= self.convergence_eps:  # Converged
+                    if diff <= self.convergence_cutoff:  # Converged
                         converged = True
                         print('Log like converged after {:d} iterations'.format(iter))
+
+            if iter == self.max_iter:
+                converged = True
+                print(f'Max iterations of {self.max_iter} reached')
             iter += 1            
 
         final_mu, final_obs_pi, final_real_pi = mu_list[-1], obs_pi_list[-1], real_pi_list[-1]
@@ -271,7 +291,7 @@ if __name__ == '__main__' and False:
     read_hist_total =  np.load(exp_path+"read_hist.npy")
 
     # with open(exp_path+"data_EM_analysis.txt", 'a') as f:
-    #     f.write("N_reads n_cpu dT mean_memory peak_memory \n")
+    #     f.write("N_reads max_procs dT mean_memory peak_memory \n")
     #     f.close()
     
     # for N_partial in np.geomspace(10000, 120000, 5).astype(np.int64):
@@ -279,10 +299,10 @@ if __name__ == '__main__' and False:
 
         N_partial = min(N_partial, bit_Vector_total.shape[0])
 
-        # for n_cpu in range(1, 11):
-        for n_cpu in [2]:
+        # for max_procs in range(1, 11):
+        for max_procs in [2]:
 
-            print("Starting experiment with {} reads and {} cpus".format(N_partial, n_cpu))
+            print("Starting experiment with {} reads and {} cpus".format(N_partial, max_procs))
             
             tracemalloc.start()
 
@@ -291,7 +311,7 @@ if __name__ == '__main__' and False:
             read_hist = copy.copy(read_hist_total[:N_partial])
             
             # Run EM and record stats
-            EM = EMclustering(bit_Vector, K=2, read_hist=read_hist, min_iter=10, n_cpus=n_cpu,
+            EM = EMclustering(bit_Vector, K=2, read_hist=read_hist, min_iter=10, max_procs=max_procs,
                             max_clusters=3, signal_thresh=0.5, info_thresh=0.5, include_g_u=True, include_del=True,
                             min_reads=10,convergence_cutoff=0.5,num_runs=100, verbose=True )
             
@@ -300,7 +320,7 @@ if __name__ == '__main__' and False:
             # Log experiment stats
             # memory_stats = [ mem/1e6 for mem in tracemalloc.get_traced_memory() ]
             # log_data = np.array((N_partial,
-            #                         n_cpu,
+            #                         max_procs,
             #                         result["dT"],
             #                         memory_stats[0], 
             #                         memory_stats[1] ))
