@@ -1,6 +1,9 @@
 """Draw DREEM output and draw conclusions.
 """
-import dreem.draw.study as study
 import dreem.draw.manipulator as manipulator
 import dreem.draw.plotter as plotter
-import dreem.draw.util as util
\ No newline at end of file
+import dreem.draw.util as util
+from dreem.draw.study import Study as Study
+from dreem.draw.load_dataset import load_dataset as load_dataset
+
+from dreem.draw.main import run, cli, params
