@@ -10,6 +10,7 @@ from svglib.svglib import svg2rlg
 from reportlab.graphics import renderPM
 from scipy.optimize import curve_fit
 import scipy
+import inspect
 
 vals = SymbolValidator().values
 
@@ -144,7 +145,7 @@ class RNAstructure(object):
         object (_type_): _description_
 
     Examples:
-        rna = RNAstructure('/Users/ymdt/src/RNAstructure/exe')
+        rna = RNAstructure('/Users/ymdt/dreem/RNAstructure/exe')
         rna.fit(sequence='ACCTTTCAGAGCTACGATCGACTAGCTAGCATCGATACAGCGACACAAGCATTTGTAGCATTAGGTCA')
         print("DeltaG + structure:", rna.predict_reference_deltaG())
         print("Ensemble energy:", rna.predict_ensemble_energy())
@@ -182,17 +183,17 @@ class RNAstructure(object):
                     out["p"]+=[float(ls[2])]
         return self.__cast_pairing_prob(out)
 
-    def draw(self, savefig, mut_rates = [], dpi=72):
+    def draw(self, savefig, sub_rate = [], dpi=72):
         self.predict_reference_deltaG()
-        if len(mut_rates):
-            mut_rates = np.array(mut_rates).reshape(-1,1)
+        if len(sub_rate):
+            sub_rate = np.array(sub_rate).reshape(-1,1)
             with open(self.s_file, 'w') as f:
-                for i in range(len(mut_rates)):
-                    f.write(f'{i+1} {mut_rates[i][0] if mut_rates[i][0] != np.nan else -999 } \n')
+                for i in range(len(sub_rate)):
+                    f.write(f'{i+1} {sub_rate[i][0] if sub_rate[i][0] != np.nan else -999 } \n')
                 f.close()
             
         cmd = self.rnastructure_path+'draw '+self.ct_file + ' --svg '+self.svg_file+' -n -1'
-        self.__run_command(cmd if len(mut_rates) == 0 else cmd+' -s '+self.s_file)
+        self.__run_command(cmd if len(sub_rate) == 0 else cmd+' -s '+self.s_file)
         svg_code = self.__read_svg()
         drawing = svg2rlg(self.svg_file)
         renderPM.drawToFile(drawing, self.png_file, fmt="PNG", dpi=dpi)
@@ -222,7 +223,7 @@ class RNAstructure(object):
         self.pfs_file = f"{temp_prefix}.pfs"
         self.ct_file = f"{temp_prefix}.ct"
         self.dot_file = f"{temp_prefix}_dot.txt"
-        self.fasta_file = temp_prefix+'.fasta'
+        self.fasta_file = temp_prefix+'fasta'
         self.prob_file = temp_prefix+'_prob.txt'
         self.svg_file = temp_prefix+'.svg'
         self.png_file = temp_prefix+'.png'
@@ -279,3 +280,49 @@ class RNAstructure(object):
             error_msg = exc.output.decode("utf8")
         return output, error_msg
 
+def save_plot(func,  *args, **kwargs):
+    """Default arguments for saving a plotly figure.
+
+    Args:
+        to_html (str, optional): File name to save the figure as a html.
+        to_png (str, optional): File name to save the figure as a png.
+    """
+
+    def wrapper(*args, **kwargs):
+        out = func(*args, **kwargs)
+        to_html, to_png = kwargs.pop('to_html', None), kwargs.pop('to_png', None)
+        if to_html:
+            out['fig'].write_html(to_html)
+        if to_png:
+            out['fig'].write_image(to_png)
+        return out
+    wrapper.__name__=func.__name__
+    wrapper.__doc__=func.__doc__
+    return wrapper   
+
+def extract_args(func):
+    """Extract the arguments of a function.
+
+    Args:
+        func (function): Function to extract the arguments from.
+
+    Returns:
+        list: List of the arguments of the function.
+    """
+    return inspect.getfullargspec(func).args
+
+
+def assert_only_one_row(df):
+    """Assert that the dataframe has only one row.
+
+    Args:
+        df (pd.DataFrame): Dataframe to check.
+
+    Raises:
+        ValueError: If the dataframe has more than one row.
+    """
+    if df.shape[0] > 1:
+        raise ValueError("The dataframe has more than one row.")
+    
+    if df.shape[0] == 0:
+        raise ValueError("The dataframe is empty.")
